#version 120

#define M_PI 3.1415926535897932384626433832795

varying vec2 UV;
varying vec4 normalVector;
varying vec3 fragPos;

uniform sampler2D texSampler;
uniform vec4 tintColor;
uniform bool texturesEnabled;

/*
	Fog
*/
uniform bool fogEnabled;
uniform vec4 fogColor;
uniform float fogMin;
uniform float fogMax;

/*
	Lighting
*/
uniform bool lightEnabled;

void main()
{
	vec4 color;
	
	if(texturesEnabled)
	{
   	  	color = texture2D( texSampler, UV ).rgba * tintColor;
    }
    else
    {
    	color = tintColor;
    }
    
    if(color.a == 0.0f) discard;
    
    if(lightEnabled)
    {
		color.rgb *= 0.10f + pow((atan(normalVector.y,normalVector.x) + M_PI ) / (2*M_PI),2);
    }
    
    if(fogEnabled)
    {
        float dist = gl_FragCoord.z / gl_FragCoord.w;
        float fogAmount = clamp((dist-fogMin)/(fogMax-fogMin),0.0f,1.0f);

        color = mix(color,fogColor,fogAmount);
    }

	gl_FragColor = color;
	
}

