#include "actor.hpp"

using namespace Application;
using namespace Engine;

void Application::Actor::Move(float timeMod)
{
    totalSpeed = sqrt(pow(speed.x,2) + pow(speed.y,2) + pow(speed.z,2));

    if(target.x > speed.x)
    {
        speed.x += speedMod.x * timeMod;
        speed.x = speed.x > target.x ? target.x : speed.x;
    }
    else if(target.x < speed.x)
    {
        speed.x -= speedMod.x * timeMod;
        speed.x = speed.x < target.x ? target.x : speed.x;
    }

    if(target.z > speed.z)
    {
        speed.z += speedMod.z * timeMod;
        speed.z = speed.z > target.z ? target.z : speed.z;
    }
    else if(target.z < speed.z)
    {
        speed.z -= speedMod.z * timeMod;
        speed.z = speed.z < target.z ? target.z : speed.z;
    }

    if(target.y != 0.0f)
    {
        if(target.y > speed.y)
        {
            speed.y += speedMod.y * timeMod;
            speed.y = speed.y > target.y ? target.y : speed.y;
        }
        else if(target.y < speed.y)
        {
            speed.y -= speedMod.y * timeMod;
            speed.y = speed.z < target.y ? target.y : speed.y;
        }
    }

    pos += speed * timeMod;
}

void Application::Actor::PointCollision(float x1, float z1,float dist)
{
    float angle = atan2(pos.z-z1,pos.x-x1);

    pos.x += cos(angle) * fabs(dist-radius);
    pos.z += sin(angle) * fabs(dist-radius);

    speed.x += cos(angle) * fabs(dist-radius);
    speed.z += sin(angle) * fabs(dist-radius);
}

void Application::Actor::GetPointCollision(float x1, float z1, float x2, float z2)
{
    float dist = sqrt(pow(pos.x-x1,2) + pow(pos.z-z1,2));

    if(dist < radius)
    {
        PointCollision(x1,z1,dist);
    }
    else
    {
        dist = sqrt(pow(pos.x-x2,2) + pow(pos.z-z2,2));
        if(dist < radius)
        {
            PointCollision(x2,z2,dist);
        }
    }

}

void Application::Actor::Update(float timeMod)
{
    OnUpdate(timeMod);
    Move(timeMod);

    canJump = false;
}

void Application::Actor::OnFloorCollision(float a, float b, float c, float d, float timeMod)
{
    if(pos.y < (-(a * pos.x + c * pos.z + d))/b+0.20f
            && pos.y > (-(a * pos.x + c * pos.z + d))/b - (0.5f+fabs(speed.y))*timeMod && speed.y < 0.0f)
    {
        pos.y = -((a * pos.x + c * pos.z + d))/b;
        speed.y = 0.0f;
        canJump = true;
    }
}

void Application::Actor::OnWallCollision(glm::vec3 A, glm::vec3 B, glm::vec3 C, float timeMod)
{
    float x1 = std::min(A.x,std::min(B.x,C.x));
    float x2 = std::max(A.x,std::max(B.x,C.x));

    float z1 = 0.0f;
    float z2 = 0.0f;

    if(x1 == x2)
    {
        z1 = std::min(A.z,std::min(B.z,C.z));
        z2 = std::max(A.z,std::max(B.z,C.z));

        x1 = (z1 == A.z ? A.x : (z1 == B.z ? B.x : (z1 == C.z ? C.x : 0.0f)));
        x2 = (z2 == A.z ? A.x : (z2 == B.z ? B.x : (z2 == C.z ? C.x : 0.0f)));
    }
    else
    {
        z1 = (x1 == A.x ? A.z : (x1 == B.x ? B.z : (x1 == C.x ? C.z : 0.0f)));
        z2 = (x2 == A.x ? A.z : (x2 == B.x ? B.z : (x2 == C.x ? C.z : 0.0f)));
    }

    if(z1 == z2)
    {
        if(fabs(pos.z-z1) < radius)
        {
            pos.z += (pos.z < z1 ? -1 : 1) * fabs(fabs(pos.z-z1)-radius);
            speed.z = 0.0;
        }
        GetPointCollision(x1,z1,x2,z2);
        return;

    }
    else if(x1 == x2)
    {
        if(fabs(pos.x-x1) < radius)
        {
            pos.x += (pos.x < x1 ? -1 : 1) * fabs(fabs(pos.x-x1)-radius);
            speed.x = 0.0;
        }
        GetPointCollision(x1,z1,x2,z2);

        return;

    }

    float a,b,c;
    float na,nb,nc;
    float cx, cy;

    a = -(z2-z1) / (x2-x1);
    b = 1;
    c = - (z1*b + x1*a);

    na = -1.0f/a;
    nb = 1.0f;
    nc = -(pos.z * nb + pos.x*na);

    cx = (nb * c - b*nc) / (na*b-a*nb);
    cy = -(na * c - a*nc) / (na*b-a*nb);

    if(cx < std::min(x1,x2) or cx > std::max(x1,x2) or cy < std::min(z1,z2) or cy > std::max(z1,z2))
    {
        GetPointCollision(x1,z1,x2,z2);
        return;
    }
    else
    {
        float dist = sqrt(pow(pos.x-cx,2) + pow(pos.z-cy,2));

        if(dist < radius)
        {
            float angle = atan2(pos.z-cy,pos.x-cx);

            pos.x = pos.x + cos(angle) * fabs(dist-radius);
            pos.z = pos.z + sin(angle) * fabs(dist-radius);

            speed.x += cos(angle) * fabs(dist-radius);
            speed.z += sin(angle) * fabs(dist-radius);
        }
    }
}

void Application::Actor::OnTriangleCollision(glm::vec3 A, glm::vec3 B, glm::vec3 C, float timeMod)
{
    glm::vec3 N = glm::normalize(glm::cross(C - A, B - A));

    if(N.y == 0)
    {


        glm::vec3 tp = pos + glm::vec3(0.0f,0.25f,0.0f);

        if(Math::IsInsideTriangleXY(tp,A,B,C) || Math::IsInsideTriangleYZ(tp,A,B,C))
        {
            OnWallCollision(A,B,C,timeMod);
        }
    }
    else
    {
        if(Math::IsInsideTriangleXZ(pos,A,B,C))
        {
            /*
            * Let's find the normal with the good ol' cross product!
            */
            glm::vec3 N = glm::normalize(glm::cross(C - A, B - A));

            float d = -(N.x * A.x + N.y * A.y + N.z * A.z);
            float a = N.x;
            float b = N.y;
            float c = N.z;

            OnFloorCollision(a,b,c,d,timeMod);
        }
    }
}
