#include "player.hpp"

using namespace Application;
using namespace Engine;

/**
* Private class methods
*/
void Application::Player::Control(float timeMod)
{
    angle.y -= Vpad::GetRightStick().x * timeMod;
    angle.x -= Vpad::GetRightStick().y * timeMod;

    angle.x = angle.x < -M_PI/2.5f ? -M_PI/2.5f : (angle.x > M_PI/2.5f ? M_PI/2.5f : angle.x);

    target.x = 0.0f;
    target.z = 0.0f;

    glm::vec2 lstick = Vpad::GetLeftStick();
    float moveAngle = atan2(lstick.y, lstick.x);
    float totalSpeed = sqrt(pow(lstick.x,2) + pow(lstick.y,2));

    target.y = -0.5f;
    if(canJump && Vpad::GetButton(0) == Pressed)
    {
        speed.y = 0.875f;
        wave = M_PI * (3.0f/4.0f);
    }

    if(Vpad::GetButton(0) == Released && speed.y > 0.0f)
    {
        speed.y /= 1.75f;
    }

    target.z = totalSpeed* sin(moveAngle-angle.y) * 0.20f;
    target.x = totalSpeed* cos(moveAngle-angle.y) * 0.20f;
}

void Application::Player::Waves(float timeMod)
{
    if(canJump)
    {
        if(totalSpeed > 0.0f)
        {
            wave += 1.25f*totalSpeed*timeMod;
        }
        else
        {
            if(sin(wave) != 0)
            {
                float oldSin = sin(wave);
                wave += 0.225f*timeMod;
                if(fabs(oldSin)/oldSin != fabs(sin(wave))/sin(wave))
                {
                    wave = 0.0f;
                }
            }
        }
    }

}

//! ------------------------ //

/**
* Public class methods
*/

Application::Player::Player(glm::vec3 nPos)
{
    startPos = nPos;
    pos = nPos;
    angle = glm::vec3(0.0f,0.0f,0.0f);

    speed = glm::vec3();
    target = speed;

    canJump = false;

    wave = 0.0f;

    speedMod = glm::vec3(0.02f,0.03f,0.02f);
}

void Application::Player::OnUpdate(float timeMod)
{
    Control(timeMod);
    Waves(timeMod);

    if(pos.y < -25.0f)
    {
        pos = startPos;
    }
}

void Application::Player::UpdateCamera(Engine::Camera1stPerson& cam)
{
    cam.SetPos(pos + glm::vec3(0.0f,2.0f + (canJump ? sin(wave)*0.5f : 0.0f),0.0f));
    cam.SetAngle(angle);
}
