#include "stage.hpp"

using namespace Application;
using namespace Engine;

void Application::Stage::GetModelCollision(Actor*ac, Engine::Model* m, glm::vec3 scale, float timeMod)
{
    std::vector<GLuint> indices = m->GetMesh(0)->GetIndexArray();
    std::vector<GLfloat> vertices = m->GetMesh(0)->GetVertexArray();

    int loop = 0;
    glm::vec3 A,B,C;
    for(unsigned int i = 0; i < indices.size(); i++)
    {
        switch(loop)
        {
        case 0: A.x = vertices[i]; break;
        case 1: A.y = vertices[i]; break;
        case 2: A.z = vertices[i]; break;
        case 3: B.x = vertices[i]; break;
        case 4: B.y = vertices[i]; break;
        case 5: B.z = vertices[i]; break;
        case 6: C.x = vertices[i]; break;
        case 7: C.y = vertices[i]; break;
        case 8: C.z = vertices[i]; break;
        default: break;
        }
        loop = loop >= 8 ? 0 : loop+1;
        if(loop == 0)
        {
            A *= scale.x;
            B *= scale.y;
            C *= scale.z;

            ac->OnTriangleCollision(A,B,C,timeMod);
        }
    }
}

void Application::Stage::InitStage(Engine::Assets* ass)
{
    mRoom = ass->GetModel("room2");
    mCylinder = ass->GetModel("cylinder");
    mSkybox = ass->GetModel("skybox");
    mSkybox->GetMesh(0)->SetClamping(true);

    bmpCityBg = ass->GetBitmap("cityBg");
    bmpSky = ass->GetBitmap("sky");
}

void Application::Stage::DrawStage(Engine::Graphics* g)
{
    g->SetTexturesEnabled(true);
    g->SetLighting(false);

    g->PushMatrix();
    g->Identity();
    g->Translate(cam.GetPos().x,cam.GetPos().y,cam.GetPos().z);
    g->Translate(0.0f,50.0f,0.0f);
    g->Scale(120.0f,120.0f,120.0f);
    g->UseTransform();
    g->DrawModel(mSkybox,bmpSky);
    g->PopMatrix();

    g->PushMatrix();
    g->Identity();
    g->Translate(cam.GetPos().x,0.0f,cam.GetPos().z);
    g->Translate(0.0f,50.0f,0.0f);
    g->Scale(100.0f,100.0f,100.0f);
    g->UseTransform();
    g->DrawModel(mCylinder,bmpCityBg);

    g->PopMatrix();

    g->SetFog(glm::vec4(0.0f,0.0f,0.0f,1.0f),7.5f,50.0f);

    g->SetLighting(true);
    g->SetTexturesEnabled(false);

    g->PushMatrix();
    g->Scale(8.0f,8.0f,8.0f);
    g->UseTransform();
    glEnable(GL_CULL_FACE);
    g->DrawModel(mRoom,nullptr);
    glDisable(GL_CULL_FACE);
    g->PopMatrix();

    g->SetTexturesEnabled(true);
    g->SetLighting(false);

    g->DisableFog();
}

void Application::Stage::UpdateStage(float timeMod)
{

}

void Application::Stage::OnActorCollision(Actor* ac, float timeMod)
{
    GetModelCollision(ac,mRoom,glm::vec3(8.0f,8.0f,8.0f),timeMod);
}
