/**
* Application
* Application core class
*
* Description: user-created application behavior
*
* (c) 2017 Jani Nykänen
*/

#ifndef __APPLICATION__BASE__
#define __APPLICATION__BASE__

#include "../Engine/Core/application_manager.hpp"
#include "../Engine/Renderer/shader.hpp"
#include "../Engine/Loader/asset_loader.hpp"

#include "control_state.hpp"
#include "vpad.hpp"
#include "config.hpp"
#include "Game/game.hpp"

namespace Application
{

/**
* Application class
*/
class AppBase : public Engine::ApplicationManager
{
private:

    /**
    * Private helper functions
    */
    void OnLoaded();
    void DrawToCanvas(Engine::Graphics* g);
    void CalculateCanvasProp();

    /**
    * Overridden methods
    */
    //! Base
    void OnInit();
    void OnStart();
    void OnDraw(Engine::Graphics* g);
    void OnUpdate(double deltaTime);
    void OnDestroy();

    //! Events
    void OnResize(int width, int height);
    void OnKeyDown(int key);
    void OnKeyUp(int key);
    void OnMouseMove(double x, double y);
    void OnMouseDown(int button);
    void OnMouseUp(int button);
    void OnMouseWheel(double offset);

    /**
    * Member variables
    */
    //! Assets
    Engine::Assets* ass;
    Engine::AssetBuffer assbuf;

    //! Logic
    float timeMod;

    //! Config
    Config config;

    //! Graphics
    Engine::Shader* shader;
    Engine::RenderTarget* canvas;
    glm::vec2 canvasPos;
    glm::vec2 canvasScale;

    //! Scenes
    Scene* sceneGame;
    Scene* currentScene;

    //! Loading
    bool isLoading;
    int loadState;
    bool loadingReady;

public:

};

}

#endif // __APPLICATION__BASE__

