#include "config.hpp"

using namespace Application;
using namespace ClumsyML;

Config Application::ReadConfigFile(std::string path)
{
    Config c;

    Document* doc = LoadDocument(path);

    std::vector< std::vector<std::string> > modules;

    c.fps = 60.0f;
    c.assetPath = "";
    c.shaderPath = "";
    c.canvasSize.x = 640;
    c.canvasSize.y = 480;
    c.defaultWinSize = Point (640,480);
    c.defaultFullscreenEnabled = false;

    for(Node* dnode : doc->GetNodes())
    {
        if(dnode->GetName() != "application") continue;

        for(Node* n : dnode->GetNodes())
        {
            for(Keypair kp : n->GetAttributes())
            {
                if(n->GetName() == "properties")
                {
                    if(kp.key == "fps")
                    {
                        c.fps = std::stoi(kp.value);
                    }
                    else if(kp.key == "assetpath")
                    {
                        c.assetPath = kp.value;
                    }
                    else if(kp.key == "shaderpath")
                    {
                        c.shaderPath = kp.value;
                    }
                }
                else if(n->GetName() == "window")
                {
                    if(kp.key == "width")
                    {
                        c.defaultWinSize.x = std::stoi(kp.value);
                    }
                    else if(kp.key == "height")
                    {
                        c.defaultWinSize.y = std::stoi(kp.value);
                    }
                    else if(kp.key == "fullscreen")
                    {
                        c.defaultFullscreenEnabled = kp.value == "true" ? true : false;
                    }
                }

                else if(n->GetName() == "canvas")
                {
                    if(kp.key == "width")
                    {
                        c.canvasSize.x = std::stoi(kp.value);
                    }
                    else if(kp.key == "height")
                    {
                        c.canvasSize.y = std::stoi(kp.value);
                    }
                    else if(kp.key == "enabled")
                    {
                        c.canvasEnabled = kp.value == "true" ? true : false;
                    }
                }
            }
        }
    }

    return c;
}
