#include "control_state.hpp"

using namespace Application;;

/**
    Static variables
**/
static std::vector<State> keystates;
static std::vector<State> mbstates;
static glm::vec2 mousePos;
static glm::vec2 mouseDelta;
static glm::vec2 csize;
static glm::vec2 dim;


//! ------------------------------------- //


/**
    Core control functions
**/

void Application::ControlState::Init()
{
    mousePos = glm::vec2(0,0);

    keystates = std::vector<State> (GLFW_KEY_LAST);
    for(auto& k : keystates)
    {
        k = Up;
    }

    mbstates = std::vector<State> (16);
    for(auto& m : mbstates)
    {
        m = Up;
    }
}

void Application::ControlState::OnKeyDown(int key)
{
    if(key < (int)keystates.size() && key >= 0 && keystates[key] == Up)
        keystates[key] = Pressed;
}

void Application::ControlState::OnKeyUp(int key)
{
    if(key < (int)keystates.size() && key >= 0  && keystates[key] == Down)
        keystates[key] = Released;
}

void Application::ControlState::OnMouseMove(double x, double y)
{
    mouseDelta = glm::vec2(x,y) - mousePos;

    mousePos.x = (float)x;
    mousePos.y = (float)y;
}

void Application::ControlState::OnMouseDown(int button)
{
    if(button < (int)keystates.size() && button >= 0 && mbstates[button] == Up)
        mbstates[button] = Pressed;
}

void Application::ControlState::OnMouseUp(int button)
{
    if(button < (int)keystates.size() && button >= 0  && mbstates[button] == Down)
        mbstates[button] = Released;
}

void Application::ControlState::Update()
{
    for(auto& k : keystates)
    {
        k = k == Pressed ? Down : (k == Released ? Up : k);
    }

    for(auto& k : mbstates)
    {
        k = (k == Pressed) ? Down : (k == Released ? Up : k);
    }

    mouseDelta = glm::vec2(0.0f,0.0f);
}

void Application::ControlState::SetProportions(glm::vec2 ndim, glm::vec2 cs)
{
    csize.x = cs.x;
    csize.y = cs.y;
    dim.x = ndim.x;
    dim.y = ndim.y;
}

State Application::ControlState::GetKeyState(int key)
{
    if(key < (int)keystates.size() && key >= 0)
        return keystates[key];

    return Up;
}

State Application::ControlState::GetMouseState(int button)
{
    if(button < (int)mbstates.size() && button >= 0)
        return mbstates[button];

    return Up;
}

glm::vec2 Application::ControlState::GetMousePos()
{
    return mousePos;
}

glm::vec2 Application::ControlState::GetRelativeMousePos()
{
    glm::vec2 rmousePos;

    rmousePos.x = (float)csize.x/(float)dim.x * mousePos.x;
    rmousePos.y = (float)csize.y/(float)dim.y * mousePos.y;

    return rmousePos;
}

glm::vec2 Application::ControlState::GetMouseDelta()
{
    return mouseDelta;
}
