#include "vpad.hpp"

using namespace Application;
using namespace Vpad;

/**
* Static variables
*/

struct Button
{
    int keycode;
    int joybutton;
    State state;
};

static glm::vec2 leftStick;
static glm::vec2 rightStick;
static std::vector<Button> buttons;

//! ------------------------- //

void Application::Vpad::Init()
{
    leftStick = glm::vec2(0.0f,0.0f);
    rightStick = glm::vec2(0.0f,0.0f);
}

void Application::Vpad::Update()
{
    leftStick.x = 0.0f;
    leftStick.y = 0.0f;

    rightStick.x = 0.0f;
    rightStick.y = 0.0f;

    if(ControlState::GetKeyState(GLFW_KEY_W) == Down)
    {
        leftStick.y = 1.0f;
    }
    else if(ControlState::GetKeyState(GLFW_KEY_S) == Down)
    {
        leftStick.y = -1.0f;
    }

    if(ControlState::GetKeyState(GLFW_KEY_A) == Down)
    {
        leftStick.x = 1.0f;
    }
    else if(ControlState::GetKeyState(GLFW_KEY_D) == Down)
    {
        leftStick.x = -1.0f;
    }

    if(fabs(leftStick.x) == fabs(leftStick.y))
    {
        leftStick *= 0.708f;
    }

    glm::vec2 md = ControlState::GetMouseDelta();

    rightStick.x = md.x/96.0f;
    rightStick.y = md.y/96.0f;

    for(auto& b : buttons)
    {
        b.state = ControlState::GetKeyState(b.keycode);
    }
}

void Application::Vpad::PushButton(int keycode, int joybutton)
{
    buttons.push_back(Button {keycode,joybutton,Down });
}

glm::vec2 Application::Vpad::GetLeftStick()
{
    return leftStick;
}

glm::vec2 Application::Vpad::GetRightStick()
{
    return rightStick;
}

State Application::Vpad::GetButton(unsigned int id)
{
    return id >= buttons.size() ? Up : buttons[id].state;
}
