/**
* Engine
* Application Manager
*
* Description: handles base application behavior
*
* (c) 2017 Jani Nykänen
*/

#ifndef __ENGINE__APPLICATION_MANAGER__
#define __ENGINE__APPLICATION_MANAGER__

#include <vector>
#include <stdexcept>
#include <iostream>

#include <GL/glew.h>
#include <GLFW/glfw3.h>
#include <GL/gl.h>

#include "events.hpp"
#include "display_handle.hpp"
#include "../Renderer/graphics.hpp"
#include "../Types/point.hpp"

namespace Engine
{

/**
* ApplicationManager class
*/
class ApplicationManager : public EventHandle, public DisplayHandle
{
private:

    /**
    * Private methods
    */
    void Initialize();
    void Update(double deltaTime);
    void Draw(Graphics* g);
    void Destroy();

    /**
    * Private utility methods
    */
    void CreateWindow();

    /**
    * Private variables
    */
    bool isRunning =false;
    bool programDestroyed =false;
    Graphics* g;
    double oldTime;
    double newTime;

    /**
    * Private overridden methods
    */
    void OnResizeEvent(int width, int height);

protected:

    /**
    * Member variables
    */
    std::vector<std::string> arguments;
    Point winSize;
    Point defWinSize;
    bool defFullscreen;

    /**
    * Virtual methods
    */
    virtual void OnInit() {};
    virtual void OnStart() {};
    virtual void OnDraw(Graphics* g) {};
    virtual void OnUpdate(double deltaTime){};
    virtual void OnDestroy() {};
    virtual void OnCloseWindow();
    virtual void OnResize(int w, int h) {};

public:

    /**
    * Public methods
    */

    //! Run application
    void Run(int argc, char ** argv);

    //! Constructor & descructor
    inline ApplicationManager() {}
    ~ApplicationManager();

    //! Other
    inline void Terminate() {isRunning=false;}
    inline Graphics* GetGraphics() { return g; }
    inline GLFWwindow* GetWindow() { return window; }
};

}

#endif // __ENGINE__APPLICATION_MANAGER
