#include "asset_loader.hpp"

using namespace Engine;
using namespace ClumsyML;

void Engine::Assets::PushBitmap(RawBitmap * bmp, std::string name)
{
    rawBitmaps.push_back(bmp);
    bmpNames.push_back(name);
}

void Engine::Assets::PushModel(Model* mdl, std::string name)
{
    models.push_back(mdl);
    mdlNames.push_back(name);
}

void Engine::Assets::PushRawModel(RawModel * mdl, std::string name)
{
    rawModels.push_back(mdl);
    mdlNames.push_back(name);
}

void Engine::Assets::PushMap(TmxParser::TmxMap * map, std::string name)
{
    maps.push_back(map);
    mapNames.push_back(name);
}

/*
    TODO: There is a lot of code repeated. Fix it. Maybe. Somehow.
*/
Bitmap* Engine::Assets::GetBitmap(std::string name)
{
    unsigned int count = 0;
    for(std::string n : bmpNames)
    {
        if(n == name)
        {
            return count <= bitmaps.size() ? bitmaps[count] : nullptr;
        }
        count++;
    }

    return nullptr;
}

Bitmap* Engine::Assets::GetBitmap(unsigned int id)
{
    return id <= bitmaps.size() ? bitmaps[id] : nullptr;
}

Model* Engine::Assets::GetModel(std::string name)
{
    unsigned int count = 0;
    for(std::string n : mdlNames)
    {
        if(n == name)
        {
            return count <= models.size() ? models[count] : nullptr;
        }
        count++;
    }

    return nullptr;
}

Model* Engine::Assets::GetModel(unsigned int id)
{
    return id <= models.size() ? models[id] : nullptr;
}

TmxParser::TmxMap* Engine::Assets::GetMap(std::string name)
{
    unsigned int count = 0;
    for(std::string n : mapNames)
    {
        if(n == name)
        {
            return count <= maps.size() ? maps[count] : nullptr;
        }
        count++;
    }

    return nullptr;
}

TmxParser::TmxMap* Engine::Assets::GetMap(unsigned int id)
{
    return id <= maps.size() ? maps[id] : nullptr;
}

void Engine::Assets::RawBitmapsToBitmaps()
{
    for(auto b : rawBitmaps)
    {
        bitmaps.push_back(RawTextureToBitmap(b));
    }
}

void Engine::Assets::RawmodelsToModels()
{
    for(auto r : rawModels)
    {
        models.push_back(RawToModel(r));
    }
}

Engine::Assets::Assets(){}

Engine::Assets::~Assets()
{
    for(Bitmap* it : bitmaps)
    {
        if(it != nullptr)
            delete it;
    }

    for(Model* it : models)
    {
        if(it != nullptr)
            delete it;
    }

    for(TmxParser::TmxMap* it : maps)
    {
        if(it != nullptr)
            delete it;
    }
}

Engine::BufferObject::BufferObject(){}

Engine::BufferObject::BufferObject(std::string otype, std::string oname, std::string opath)
{
    type = otype;
    name = oname;
    path = opath;
}

Engine::AssetBuffer::AssetBuffer(Assets* _assPointer)
{
    assPointer = _assPointer;
    index = 0;
    maxIndex = 0;

}

Engine::AssetBuffer::AssetBuffer()
{
    index = 0;
    maxIndex = 0;
    assPointer = nullptr;

}

void Engine::AssetBuffer::PushObject(std::string otype, std::string oname, std::string opath)
{
    bufobjs.push_back(new BufferObject(otype,oname,opath));

    maxIndex++;
}

bool Engine::AssetBuffer::LoadNextElement()
{
    if(maxIndex == 0 || index > maxIndex) return true;

    std::string name = bufobjs[index]->name;
    std::string type = bufobjs[index]->type;
    std::string path = bufobjs[index]->path;

    if(type == "bitmap")
    {
        RawBitmap* bmp = LoadRawTexture(path);
        assPointer->PushBitmap(bmp,name);
    }
    else if(type == "model")
    {
        RawModel * mdl = LoadRawModel(path);
        assPointer->PushRawModel(mdl,name);
    }
    else if(type == "map")
    {
        TmxParser::TmxMap * map = TmxParser::LoadTmxMap(path);
        if(map == nullptr)
        {
            throw std::runtime_error(TmxParser::GetError());
        }
        assPointer->PushMap(map,name);
    }

    index++;

    return index == maxIndex;
}

unsigned int Engine::AssetBuffer::GetSize()
{
    return maxIndex;
}

AssetBuffer Engine::CreateAssetBuffer(std::string _path,Assets* assPointer)
{
    AssetBuffer buf = AssetBuffer(assPointer);

    Document * doc = LoadDocument(_path);

    std::string workdir;

    std::string type;
    std::string path;
    std::string name;

    for(Node* dnode : doc->GetNodes())
    {
        if(dnode->GetName() != "list") continue;

        for(auto kp : dnode->GetAttributes())
        {
            if(kp.key == "workdir") workdir = kp.value + "/";
        }

        for(Node* n : dnode->GetNodes())
        {
            type = n->GetName();

            for(auto kp : n->GetAttributes())
            {
                if(kp.key == "name") name = kp.value;
                else if(kp.key == "path") path = workdir + kp.value;

            }

            if(type == "bitmap" || type == "model" || type == "map") buf.PushObject(type,name,path);
        }
    }

    return buf;
}


/*
    Threading fun, load assest in the background!
*/
static std::thread tt;
static std::mutex mtx;
static std::string err;
static int status;

static int t_Load(Engine::AssetBuffer* buf)
{
    status = 0;
    try
    {
        while(!buf->LoadNextElement());
    }
    catch(std::runtime_error e)
    {
        err = e.what();
        status = -1;
        return 0;
    }

    status = 1;

    return 0;
}

void Engine::LoadAssets(Engine::AssetBuffer* buf)
{
    status = 0;

    tt = std::thread(t_Load,buf);
}

int Engine::IsAssetLoadingReady()
{
    std::lock_guard<std::mutex> guard(mtx);
    return status;
}

void Engine::JoinAssetLoaderThread()
{
    if(tt.joinable()) tt.join();
}

std::string Engine::GetAssetLoaderError()
{
    return err;
}
