
/**
* Engine
* Asset loader
*
* (c) 2017 Jani Nykänen
*/

#ifndef __ENGINE__ASSET_LOADER__
#define __ENGINE__ASSET_LOADER__

#include <string>
#include <stdexcept>
#include <vector>

#include <thread>
#include <mutex>

#include "../../Library/clumsy-ml.hpp"
#include "../Renderer/model.hpp"
#include "../Renderer/bitmap.hpp"
#include "../../Library/tmxparser.hpp"

namespace Engine
{

class Assets
{
private:

    std::vector<RawBitmap*> rawBitmaps;
    std::vector<Bitmap*> bitmaps;
    std::vector<std::string> bmpNames;

    std::vector<TmxParser::TmxMap*> maps;
    std::vector<std::string> mapNames;

    std::vector<Model*> models;
    std::vector<RawModel*> rawModels;
    std::vector<std::string> mdlNames;

public:

    Bitmap* GetBitmap(std::string name);
    Bitmap* GetBitmap(unsigned int id);

    Model* GetModel(std::string name);
    Model* GetModel(unsigned int id);

    TmxParser::TmxMap* GetMap(std::string name);
    TmxParser::TmxMap* GetMap(unsigned int id);

    void PushBitmap(RawBitmap * bmp, std::string name);
    void PushModel(Model* mdl, std::string name);
    void PushRawModel(RawModel * mdl, std::string name);
    void PushMap(TmxParser::TmxMap* map, std::string name);

    void RawBitmapsToBitmaps();
    void RawmodelsToModels();

    Assets();
    ~Assets();

};

struct BufferObject
{
    std::string name;
    std::string type;
    std::string path;

    BufferObject();
    BufferObject(std::string otype, std::string oname, std::string opath);
};

class AssetBuffer
{
private:

    std::vector<BufferObject*> bufobjs;

    unsigned int index=0;
    unsigned int maxIndex = 0;

    Assets * assPointer; // Eheheheheh

public:

    AssetBuffer();
    AssetBuffer(Assets* assetPointer);

    void PushObject(std::string otype, std::string oname, std::string opath);
    bool LoadNextElement();

    unsigned int GetSize();
    inline unsigned int GetIndex() {return index;}
};

AssetBuffer CreateAssetBuffer(std::string path,Assets* assPointer);

void LoadAssets(Engine::AssetBuffer* buf);
int IsAssetLoadingReady();
void JoinAssetLoaderThread();
std::string GetAssetLoaderError();

}

#endif // __ENGINE__ASSET_LOADER__
