#include "bitmap.hpp"

using namespace Engine;

/**
* Bitmap class members
*/

Engine::Bitmap::Bitmap(GLuint ntex, int w, int h)
{
    texture = ntex;
    size = Point(w,h);
}

//! --------------------------- //

/**
* Bitmap loading & conversion functions
*/

RawBitmap* Engine::LoadRawTexture(std::string path)
{
    RawBitmap* rw = new RawBitmap();

    rw->data = SOIL_load_image(path.c_str(),&rw->width,&rw->height,&rw->channels,SOIL_LOAD_AUTO);
    if(rw->data == nullptr)
    {
        throw std::runtime_error("Failed to load a bitmap in " + path);
    }

    return rw;
}

Bitmap * Engine::LoadBitmap(std::string path)
{
    GLuint tex = SOIL_load_OGL_texture
	(
		path.c_str(),
		SOIL_LOAD_AUTO,
		SOIL_CREATE_NEW_ID,
		SOIL_FLAG_MIPMAPS | SOIL_FLAG_COMPRESS_TO_DXT
	);

	if(tex == 0)
    {
        throw std::runtime_error("Failed to convert a raw bitmap data to a texture!");
    }

    RawBitmap* rw = new RawBitmap;
    rw->data = SOIL_load_image(path.c_str(),&rw->width,&rw->height,&rw->channels,SOIL_LOAD_AUTO);
    if(rw->data == nullptr)
    {
        throw std::runtime_error("Failed to load a bitmap in " + path);
    }

    int width = rw->width;
    int height = rw->height;

    delete rw;

    return new Bitmap(tex,width,height);
}

Bitmap * Engine::RawTextureToBitmap(RawBitmap* rw)
{
    if(rw->data == nullptr) return nullptr;

    GLuint tex =0;

    glGenTextures(1,&tex);
    glBindTexture(GL_TEXTURE_2D, tex);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, rw->width, rw->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, rw->data);

    if(tex == 0)
    {
        throw std::runtime_error("Failed to convert a raw bitmap data to a texture!");
    }

    int width = rw->width;
    int height = rw->height;

    delete rw;

    return new Bitmap(tex,width,height);
}
