#include "effects.hpp"

using namespace Engine;

/**
*  Public class methods
*/

void Engine::Effect::InitEffects(Shader* sh)
{
    GLuint program = sh->GetProgram();

    colorID = program != 0 ? glGetUniformLocation(program, "tintColor") : 0;
    fogEnabledID = program != 0 ? glGetUniformLocation(program, "fogEnabled") : 0;
    fogColorID = program != 0 ? glGetUniformLocation(program, "fogColor") : 0;
    fogMaxID = program != 0 ? glGetUniformLocation(program, "fogMax") : 0;
    fogMinID = program != 0 ? glGetUniformLocation(program, "fogMin") : 0;
    pLimiterID = program != 0 ? glGetUniformLocation(program, "paletteLimiter") : 0;
    texEnabledID = program != 0 ? glGetUniformLocation(program,"texturesEnabled") : 0;
    lightStateID = program != 0 ? glGetUniformLocation(program,"lightEnabled") : 0;
}

void Engine::Effect::ResetEffects()
{
    glUniform4fv(colorID, 1, &glm::vec4(1.0f,1.0f,1.0f,1.0f)[0]);

    glUniform4fv(fogColorID, 1, &glm::vec4(1.0f,1.0f,1.0f,1.0f)[0]);
    glUniform1f(fogMaxID,1.0f);
    glUniform1f(fogMinID,0.5f);
    glUniform1i(fogEnabledID,false);
    glUniform1i(texEnabledID,true);
    glUniform1i(lightStateID,false);
}

void Engine::Effect::SetColor(float r, float g, float b, float a)
{
    glUniform4fv(colorID, 1, &glm::vec4(r,g,b,a)[0]);
}

void Engine::Effect::DisableFog()
{
    glUniform1i(fogEnabledID,false);
}

void Engine::Effect::SetFog(glm::vec4 color,float min, float max)
{
    glUniform4fv(fogColorID, 1, &color[0]);
    glUniform1f(fogMaxID,max);
    glUniform1f(fogMinID,min);
    glUniform1i(fogEnabledID,true);
}

bool Engine::Effect::SetTexturesEnabled(bool state)
{
    bool ret = oldState;

    glUniform1i(texEnabledID,state ? 1 : 0);
    oldState = state;

    return ret;
}

void Engine::Effect::SetLighting(bool state)
{
    glUniform1i(lightStateID,state ? 1 : 0);
}
