/**
* Engine
* Effects (tinting, fog etc)
*
* (c) 2017 Jani Nykänen
*
*/

#ifndef __ENGINE__EFFECTS__
#define __ENGINE__EFFECTS__

#include <iostream>
#include <string>
#include <vector>

#include <GL/glew.h>
#include <GL/glext.h>
#include <GL/gl.h>
#include <GL/glu.h>

#define GLM_FORCE_RADIANS
#include <glm/glm.hpp>
#include <glm/gtc/matrix_transform.hpp>

#include "shader.hpp"
#include "../Types/color.hpp"

namespace Engine
{

class Effect
{
private:

    /**
    * Private variables
    * Mostly IDs
    */
    GLuint colorID;
    GLuint fogEnabledID;
    GLuint fogMaxID;
    GLuint fogMinID;
    GLuint fogColorID;
    GLuint pLimiterID;
    GLuint texEnabledID;
    GLuint lightStateID;

    bool oldState;

public:

    void InitEffects(Shader* sh);

    /**
    * Effect manipulation
    */
    void ResetEffects();
    void SetColor(float r,float g,float b,float a);
    inline void SetColor(Color c) { SetColor(c.r,c.b,c.g,c.a); }
    void DisableFog();
    void SetFog(glm::vec4 color,float min, float max);
    bool SetTexturesEnabled(bool state);
    void SetLighting(bool state);

};

}

#endif // __ENGINE__EFFECTS__
