#include "mesh.hpp"

/**
* Public class methods (they are all public)
*/

Engine::Mesh::Mesh()
{
    clamp = false;
}

Engine::Mesh::Mesh(std::vector<GLfloat> vArr, std::vector<GLuint> indArr, std::vector<GLfloat> texArr, std::vector<GLfloat> nrmArr)
{
    glGenBuffers(1,&vArrObj);
    glGenBuffers(1,&iArrObj);
    glGenBuffers(1,&tArrObj);
    glGenBuffers(1,&nArrObj);

    vertexArray = vArr;
    indexArray = indArr;
    texCoordArray = texArr;
    normalArray = nrmArr;

    glBindBuffer(GL_ARRAY_BUFFER, vArrObj);
    glBufferData(GL_ARRAY_BUFFER, vertexArray.size()* sizeof(GLfloat), &vertexArray[0], GL_STATIC_DRAW);

    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, iArrObj);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, indexArray.size()* sizeof(GLuint), &indexArray[0], GL_STATIC_DRAW);

    glBindBuffer(GL_ARRAY_BUFFER, tArrObj);
    glBufferData(GL_ARRAY_BUFFER, texCoordArray.size()* sizeof(GLfloat), &texCoordArray[0], GL_STATIC_DRAW);

    glBindBuffer(GL_ARRAY_BUFFER, nArrObj);
    glBufferData(GL_ARRAY_BUFFER, normalArray.size()* sizeof(GLfloat), &normalArray[0], GL_STATIC_DRAW);

    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

    clamp = false;
}

void Engine::Mesh::Draw(Bitmap* bmp, bool linFilter)
{
    Draw(bmp == nullptr ? 0 : bmp->GetTexture(),linFilter);
}

void Engine::Mesh::Draw(GLuint tex,bool linFilter)
{
    glEnable(GL_TEXTURE_2D);

    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D,tex);

    if(clamp)
    {
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
    }
    else
    {
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    }

    if(linFilter)
    {
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    }
    else
    {
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    }

    glEnableVertexAttribArray(0);
    glBindBuffer(GL_ARRAY_BUFFER, vArrObj);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, iArrObj);
    glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, (GLvoid*)0);

    glEnableVertexAttribArray(1);
    glBindBuffer(GL_ARRAY_BUFFER, tArrObj);
    glVertexAttribPointer(1,2,GL_FLOAT,GL_FALSE, 0,(GLvoid*)0);

    glEnableVertexAttribArray(2);
    glBindBuffer(GL_ARRAY_BUFFER, nArrObj);
    glVertexAttribPointer(2,3, GL_FLOAT,GL_FALSE,0,(GLvoid*)0);

    glDrawElements(GL_TRIANGLES, indexArray.size(), GL_UNSIGNED_INT, 0);
    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

    glDisableVertexAttribArray(0);
    glDisableVertexAttribArray(1);
    glDisableVertexAttribArray(2);

}

void Engine::Mesh::SetClamping(bool value)
{
    clamp = value;
}

void Engine::Mesh::NewVertices(std::vector<GLfloat> nvert)
{
    vertexArray = nvert;

    glBindBuffer(GL_ARRAY_BUFFER, vArrObj);
    glBufferSubData(GL_ARRAY_BUFFER,0, vertexArray.size()* sizeof(GLfloat), &vertexArray[0]);
    glBindBuffer(GL_ARRAY_BUFFER, 0);
}

void Engine::Mesh::NewUVCoords(std::vector<GLfloat> nuc)
{
    texCoordArray = nuc;

    glBindBuffer(GL_ARRAY_BUFFER, tArrObj);
    glBufferSubData(GL_ARRAY_BUFFER,0, texCoordArray.size()* sizeof(GLfloat), &texCoordArray[0]);
    glBindBuffer(GL_ARRAY_BUFFER, 0);
}
