#include "model.hpp"

using namespace Engine;

Engine::Model::Model()
{

}

Engine::Model::Model(std::vector<Mesh*> mArr)
{
    meshes = mArr;
}

void Engine::Model::PushMesh(Mesh* m)
{
    meshes.push_back(m);
}

void Engine::Model::PopMesh()
{
    if(meshes.size() <= 0) return;

    delete meshes[meshes.size()-1];
    meshes.pop_back();
}

void Engine::Model::Draw(std::vector<Bitmap*> bmpArray, bool linFilter)
{
    int i = 0;
    for(auto m : meshes)
    {
        m->Draw(bmpArray[i],linFilter);
        i++;
    }
}

void Engine::Model::Draw(Bitmap* bmp, bool linFilter)
{
    for(auto m : meshes)
    {
        m->Draw(bmp,linFilter);
    }
}

Engine::Mesh* Engine::Model::GetMesh(unsigned int id)
{
    return  (id < meshes.size()) ? meshes[id] : nullptr;
}

Engine::Model::~Model()
{
    for(auto m : meshes)
    {
        delete m;
    }
}

RawModel * Engine::LoadRawModel(std::string path)
{
    tinyobj::attrib_t attrib;
    std::vector<tinyobj::shape_t> shapes;
    std::vector<tinyobj::material_t> materials;
    std::string err;

    bool ret = tinyobj::LoadObj(&attrib,&shapes,&materials,&err,path.c_str());

    if(!ret)
    {
        throw std::runtime_error("Error loading a file in " + path + ":" + err);
    }



    RawModel * mdl = new RawModel;
    RawMesh* m;

    /*
        Slightly edited version of the code example int tinyobj loader's GitHub page
    */
    for (size_t s = 0; s < shapes.size(); s++)
    {
        m = new RawMesh();
        size_t ioff = 0;
        for (size_t f = 0; f < shapes[s].mesh.num_face_vertices.size(); f++)
        {
            int fv = shapes[s].mesh.num_face_vertices[f];

            for (size_t v = 0; v < (size_t)fv; v++)
            {
                tinyobj::index_t idx = shapes[s].mesh.indices[ioff + v];

                float vx = attrib.vertices[3*idx.vertex_index+0];
                float vy = attrib.vertices[3*idx.vertex_index+1]; vy = floor(vy*1000.0f) / 1000.0f;
                float vz = attrib.vertices[3*idx.vertex_index+2];

                float nx = attrib.normals[3*idx.normal_index+0];
                float ny = attrib.normals[3*idx.normal_index+1];
                float nz = attrib.normals[3*idx.normal_index+2];

                float tx = attrib.texcoords[2*idx.texcoord_index+0];
                float ty = attrib.texcoords[2*idx.texcoord_index+1];

                m->vArr.push_back(vx);m->vArr.push_back(vy);m->vArr.push_back(vz);
                m->nArr.push_back(nx); m->nArr.push_back(ny); m->nArr.push_back(nz);
                m->tArr.push_back(tx); m->tArr.push_back(ty);

                for(int i = 0; i < 3; i++)
                {
                    m->iArr.push_back(m->iArr.size());
                }

            }
            ioff += fv;
            shapes[s].mesh.material_ids[f];

            mdl->meshes.push_back(m);
        }
    }

    return mdl;
}

Model * Engine::RawToModel(RawModel* m)
{
    Model* mdl = new Model();

    for(auto ms : m->meshes)
    {
        mdl->PushMesh(new Mesh(ms->vArr,ms->iArr,ms->tArr,ms->nArr));
    }

    delete m;

    return mdl;
}
