#include "partition.hpp"

using namespace Engine;

/**
* Partition
*/
void Engine::Partition::Generate()
{
    glGenBuffers(1,&vArrObj);
    glGenBuffers(1,&iArrObj);
    glGenBuffers(1,&nArrObj);

    for(int y = 0; y < size.y; y++)
    {
        for(int x = 0; x < size.x; x++)
        {
            tArrObjs.push_back(0);
            glGenBuffers(1,&tArrObjs[y*size.x+x]);
        }
    }

    vertexArray = std::vector<GLfloat>{0.0f,0.0f,0.0f, 1.0f,0.0f,0.0f, 1.0f,1.0f,0.0f, 0.0f,1.0f,0.0f};
    indexArray = std::vector<GLuint>{0,1,2, 2,3,0};
    normalArray = std::vector<GLfloat> {0.0f,0.0f,1.0f, 0.0f,0.0f,1.0f, 0.0f,0.0f,1.0f, 0.0f,0.0f,1.0f};

    glBindBuffer(GL_ARRAY_BUFFER, vArrObj);
    glBufferData(GL_ARRAY_BUFFER, vertexArray.size()* sizeof(GLfloat), &vertexArray[0], GL_STATIC_DRAW);

    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, iArrObj);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, indexArray.size()* sizeof(GLuint), &indexArray[0], GL_STATIC_DRAW);

    glBindBuffer(GL_ARRAY_BUFFER, nArrObj);
    glBufferData(GL_ARRAY_BUFFER, normalArray.size()* sizeof(GLfloat), &normalArray[0], GL_STATIC_DRAW);

    float sx = 0.0f;
    float sy = 0.0f;
    float w = 1.0f / (float) size.x;
    float h = 1.0f / (float) size.y;

    std::vector<GLfloat> texCoords;

    for(int y = 0; y < size.y; y++)
    {
        sx = 0;
        for(int x = 0; x < size.x; x++)
        {
            texCoords =
            {
                sx,sy, sx+w,sy, sx+w,sy+h, sx,sy+h,
            };

            glBindBuffer(GL_ARRAY_BUFFER, tArrObjs[y*size.x+x]);
            glBufferData(GL_ARRAY_BUFFER, texCoords.size()* sizeof(GLfloat), &texCoords[0], GL_STATIC_DRAW);

            sx += w;
        }


        sy += h;


    }

    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
}

Engine::Partition::Partition(Point nsize)
{
    size = nsize;
    Generate();
}

void Engine::Partition::Draw(Bitmap* bmp, unsigned int column, unsigned int row, bool linFilter)
{
    GLuint tex = bmp->GetTexture();

    glEnable(GL_TEXTURE_2D);

    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D,tex);

    if(linFilter)
    {
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    }
    else
    {
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    }

    glEnableVertexAttribArray(0);
    glBindBuffer(GL_ARRAY_BUFFER, vArrObj);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, iArrObj);
    glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, (GLvoid*)0);

    glEnableVertexAttribArray(1);
    glBindBuffer(GL_ARRAY_BUFFER, tArrObjs[row*size.x+column]);
    glVertexAttribPointer(1,2,GL_FLOAT,GL_FALSE, 0,(GLvoid*)0);

    glEnableVertexAttribArray(2);
    glBindBuffer(GL_ARRAY_BUFFER, nArrObj);
    glVertexAttribPointer(2,3, GL_FLOAT,GL_FALSE,0,(GLvoid*)0);

    glDrawElements(GL_TRIANGLES, indexArray.size(), GL_UNSIGNED_INT, 0);
    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

    glDisableVertexAttribArray(0);
    glDisableVertexAttribArray(1);
    glDisableVertexAttribArray(2);
}

//! ------------------------------- //

/**
* Partition handle
*/

void Engine::PartitionHandle::InitPartitionHandle()
{
    /*
    * Generate default 16x16 partition
    */
    partitions.push_back(new Partition(Point(16,16)));
}

void Engine::PartitionHandle::CreatePartition(Point nsize)
{
    for(auto p : partitions)
    {
        if(p->GetSize() == nsize)
        {
            return;
        }
    }

    partitions.push_back(new Partition(nsize));
}


Partition* Engine::PartitionHandle::GetPartition(Point size)
{
    for(auto p : partitions)
    {
        if(p->GetSize() == size)
        {
            return p;
        }
    }
    CreatePartition(size);

    return partitions[partitions.size()-1];
}
