/**
* Engine
* Render Utility
* Description: for drawing meshes etc.
*
* (c) 2017 Jani Nykänen
*/

#ifndef __ENGINE__RENDER_UTILITY__
#define __ENGINE__RENDER_UTILITY__

#include <math.h>

#include "model.hpp"
#include "transform.hpp"
#include "effects.hpp"
#include "partition.hpp"
#include "rendertarget.hpp"

#include "../Types/color.hpp"

namespace Engine
{

class RenderUtility : public PartitionHandle, public Transform, public Effect
{
private:

    Partition* fontPart;
    Mesh* mBmpFull; //! For drawing full images
    Mesh* mBmpFullFlipY; //! For drawing render targerts...
    Mesh* mBmpReg; //! For drawing image regions
    Mesh* mTriangle; //! For drawing... triangles!

    bool linFilter;

protected:

    Point winSize;

public:

    /**
    * Model & mesh rendering
    */
    void DrawMesh(Engine::Mesh* m, Bitmap* bmp);
    void DrawMesh(Engine::Mesh* m, GLuint tex);
    void DrawModel(Engine::Model* m, Bitmap* bmp);
    void DrawModel(Engine::Model* m, std::vector<Bitmap*> bitmaps);

    /**
    * Text Rendering
    */
    void DrawText(Bitmap* font, std::string text, float x, float y);

    /**
    * 2D image rendering
    */
    void DrawBitmap(Bitmap* img, float x, float y, float scalex = 1.0f, float scaley = 1.0f);
    void DrawBitmap(RenderTarget* rt, float x, float y, float scalex = 1.0f, float scaley = 1.0f);
    void DrawBitmapRegion(Bitmap* img, float sx, float sy, float sw, float sh, float dx, float dy, float dw, float dh);

    /**
    * Shapes
    */
    void DrawTriangle(float x1, float y1, float x2, float y2, float x3, float y3);

    /**
    * Render targets
    */
    void SetRenderTarget(RenderTarget* rt);
    inline void SetRenderTarget() {SetRenderTarget(nullptr); };

    /**
    * Misc
    */
    void Clear(Color c);
    void SetDepthTest(bool mode);
    void SetBlending(bool mode);
    void SetLinearFilter(bool mode);
    inline bool GetLinearFilterEnabled() { return linFilter; }

    /**
    * Init
    */
    void InitRenderUtility();

};

};

#endif // __ENGINE__RENDER_UTILITY__
