#include "rendertarget.hpp"

using namespace Engine;

Engine::RenderTarget::RenderTarget(int w, int h)
{
    width = w;
    height = h;

    glGenFramebuffers(1, &frameBuffer);
    glBindFramebuffer(GL_FRAMEBUFFER, frameBuffer);

    glGenTextures( 1, &texture );
    glBindTexture(GL_TEXTURE_2D, texture);
    glTexImage2D(GL_TEXTURE_2D, 0,GL_RGBA, w,h, 0,GL_RGBA, GL_UNSIGNED_BYTE, 0);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

    glGenRenderbuffers(1, &depthBuffer);
    glBindRenderbuffer(GL_RENDERBUFFER, depthBuffer);
    glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT, w, h);
    glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, depthBuffer);

    glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,GL_TEXTURE_2D, texture, 0);
    glDrawBuffers(1, &std::vector<GLenum>{GL_COLOR_ATTACHMENT0}[0]);

    if(glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
    {
        throw std::runtime_error("Failed to create a framebuffer!");
    }

    glBindFramebuffer(GL_FRAMEBUFFER,0);
}

void Engine::RenderTarget::UseFrameBuffer()
{
    glBindFramebuffer(GL_FRAMEBUFFER, frameBuffer);
    glViewport(0,0,width,height);
}

Point Engine::RenderTarget::GetSize()
{
    return Point(width,height);
}

GLuint Engine::RenderTarget::GetTexture()
{
    return texture;
}

Engine::RenderTarget * Engine::CreateRenderTarget(int width, int height)
{
    return new RenderTarget(width,height);
}
