/**
* Engine
* Transformations
*
* (c) 2017 Jani Nykänen
*/
#ifndef __ENGINE__TRANSFORM__
#define __ENGINE__TRANSFORM__

#include <GL/glew.h>
#include <GL/glext.h>
#include <GL/gl.h>
#include <GL/glu.h>

#define GLM_FORCE_RADIANS
#include <glm/glm.hpp>
#include <glm/gtc/matrix_transform.hpp>

#include <vector>

#include "../Types/point.hpp"
#include "shader.hpp"

#define M_PI 3.14159265359

namespace Engine
{

struct RotateData
{
    float angle=0.0f;
    glm::vec3 axes;

    inline RotateData() {angle=0.0f;axes=glm::vec3();};
    inline RotateData(float na, glm::vec3 naxes) {angle=na;axes=naxes;};
};

class Transform
{
private:

    glm::mat4 model;
    glm::mat4 projection;
    glm::mat4 view;

    std::vector<glm::mat4> modelStorage;
    std::vector<glm::mat4> viewStorage;
    std::vector<glm::mat4> projectionStorage;

    int currentMatrix;

    GLuint modelID;
    GLuint viewID;
    GLuint projectionID;

public:

    void InitTransforms(Shader* sh);

    /**
    * Basic transformations
    */
    void Identity();
    void Scale(float x,float y,float z);
    void Translate(float x,float y,float z);
    void Rotate(float angle,glm::vec3 axis);
    void RotateMultiple(std::vector<RotateData> rotations);
    void View(glm::vec3, glm::vec3, glm::vec3);
    void View(glm::mat4);
    void Projection(float FOV, Point frame, float _min, float _max);
    void Ortho2D(float,float);
    void UseTransform();

    /**
    * Behave like glPushMatrix and glPopMatrix
    */
    void PushMatrix();
    void PopMatrix();

    void ResetBufferIndex();

};

}

#endif // __ENGINE__TRANSFORM__

