#include "clumsy-ml.hpp"

using namespace ClumsyML;

std::vector<std::string> _ParseSemicolons(std::string s)
{
    bool quotation = false;
    std::vector<std::string> buf;
    int start = 0;

    if(s.substr(0,1) == "#")
    {
        buf.push_back("");
        return buf;
    }

    for(auto i = 0; i < (int)s.length(); i++)
    {
        if(s.substr(i,1) == "" && i != (int)s.length()-1)
        {
            quotation = !quotation;
            continue;
        }

        if(s.substr(i,1) == ";" || i == (int)s.length()-1)
        {
            buf.push_back(s.substr(start,i-start + (i == (int)s.length()-1 ? 1 : 0)));
            start = i+1;
        }
    }

    return buf;
}

static std::vector<std::string> _ReadFile(std::string path) // Why did I make this static? Don't remember any more...
{
    std::vector<std::string> out;

    std::ifstream file(path.c_str());
    std::string line;

    if(file.is_open() == false)
    {
        return out;
    }

    while (std::getline(file, line))
    {
        line.erase(std::remove(line.begin(), line.end(), '\t'), line.end());

        if(line.empty() == false)
        {

            std::vector<std::string> buf = _ParseSemicolons(line);
            for(auto &s : buf)
            {
                s.erase(std::remove(s.begin(), s.end(), '\t'), s.end());
                if(s.empty() == false)
                {
                    out.push_back(s);
                }
            }
        }
    }

    return out;
}

std::vector<std::string> _ParseWords(std::string line)
{
    std::vector<std::string> out;
    std::string word;

    bool wordStarted = false;
    bool stringStarted = false;
    bool firstLetterParsed = false;

    for(unsigned int i = 0; i < line.length(); i++)
    {
        if(line[i] == '\"')
        {
            stringStarted = !stringStarted;
        }

        if(stringStarted == false)
        {
            if(isspace(line[i]))
            {
                if(wordStarted)
                {
                    if(isspace(word[0])) word.erase(0,1);
                    word.erase(remove( word.begin(), word.end(), '\"' ),word.end());

                    out.push_back(word);
                    word = "";
                    wordStarted = false;
                }
                else
                {
                    continue;
                }
            }
        }

        if(firstLetterParsed == false)
        {
            if(line[i] == '#')
            {
                out.push_back("#");
                return out;
            }
        }
        firstLetterParsed = true;

        wordStarted = true;
        word += (char)line[i];
    }

    if(wordStarted)
    {
        if(isspace(word[0])) word.erase(0,1);
        word.erase(remove( word.begin(), word.end(), '\"' ),word.end());
        out.push_back(word);
    }

    return out;
}

void _ParseLine(std::vector<std::string> line, Document* doc)
{
    if(line.size() == 0) return;

    std::string fw = line[0];

    if(fw == "#")
    {

    }
    else if(fw == "{")
    {

        if(doc->GetCurrentNode() == nullptr)
        {
            doc->SetCurrentNode(doc->PushNode(line.size() >= 2 ? line[1] : "") );
        }
        else
        {
            doc->SetCurrentNode(doc->GetCurrentNode()-> PushNode(line.size() >= 2 ? line[1] : ""));
        }
    }
    else if(fw == "}")
    {
        doc->UsePreviousNode();
    }
    else if(fw == "!")
    {
        if(line.size() < 3 || doc->GetCurrentNode() == nullptr ) return;

        doc->GetCurrentNode() ->PushAttribute(Keypair(line[1],line[2]));
    }
}

ClumsyML::Keypair::Keypair()
{

}

ClumsyML::Keypair::Keypair(std::string nkey, std::string nvalue)
{
    key = nkey;
    value = nvalue;
}

ClumsyML::Node::Node()
{

}

ClumsyML::Node::~Node()
{
    for(Node* n : nodes)
    {
        if(n == nullptr) delete n;
    }
}

ClumsyML::Node::Node(std::string n)
{
    name = n;
}

std::vector<Node*>  ClumsyML::Node::GetNodes()
{
    return nodes;
}

std::vector<Keypair>  ClumsyML::Node::GetAttributes()
{
    return attributes;
}

Node* ClumsyML::Node::PushNode(std::string name)
{
    Node* p = new Node(name);

    nodes.push_back(p);

    return p;
}

void ClumsyML::Node::PushAttribute(Keypair kp)
{
    attributes.push_back(kp);
}

std::string  ClumsyML::Node::GetName()
{
    return name;
}

ClumsyML::Document::Document()
{
    currentNode = nullptr;
    lastNodes = std::vector<Node*> ();
}

ClumsyML::Document::~Document()
{
    for(Node* n : nodes)
    {
        if(n == nullptr) delete n;
    }
}

Node* ClumsyML::Document::PushNode(std::string name)
{
    Node* p = new Node(name);

    nodes.push_back(p);

    return p;
}

std::vector<Node*> ClumsyML::Document::GetNodes()
{
    return nodes;
}

void ClumsyML::Document::SetCurrentNode(Node * n)
{
    lastNodes.push_back(currentNode);
    currentNode = n;
}

Node* ClumsyML::Document::GetCurrentNode()
{
    return currentNode;
}

void ClumsyML::Document::UsePreviousNode()
{
    currentNode = GetPreviousNode();
}

Node* ClumsyML::Document::GetPreviousNode()
{
    Node* n = lastNodes[lastNodes.size()-1];
    lastNodes.pop_back();

    return n;
}

Document* ClumsyML::LoadDocument(std::string path)
{
    std::vector<std::string> fileText = _ReadFile(path);

    Document* doc = new Document();

    std::vector<std::string> line;
    int index = 0;
    for(auto l : fileText)
    {
        line = _ParseWords(l);
        if(index == 0)
        {
            if(line.size() < 2 || line[0] != "@")
            {
                throw std::runtime_error("Header missing in " + path + "!");
            }
            else
            {
                if(line[1] != "clumsy-ml-v-020")
                {
                    throw std::runtime_error("Unknown version, only 'clumsy-ml-v-020' is supported by the parser!");
                }
            }
        }
        _ParseLine(line,doc);
        index++;
    }

    return doc;
}
