/*
    Tmx parser

    (c) 2017 Jani Nykänen
*/

#ifndef __TMX_PARSER__
#define __TMX_PARSER__

#include <rapidxml/rapidxml.hpp>
#include <vector>
#include <string>
#include <iostream>
#include <fstream>

namespace TmxParser
{

struct TmxLayer
{
    std::vector<unsigned int> data;
    std::string name;
};

struct KeyValuePair
{
    std::string key;
    std::string value;
};


class TmxMap
{
private:

    std::vector<TmxLayer> layers;
    std::vector<KeyValuePair> properties;

    unsigned int width;
    unsigned int height;

public:

    TmxMap();
    TmxMap(unsigned int nw, unsigned int nh);

    void PushLayer(TmxLayer l);
    void PushProperty(KeyValuePair prop);

    /*
        What did you say, those "inline" keywords aren't required?
        Maybe not, but they make it easier to read.

        ...and they won't hurt anyone. I guess. I hope...
    */
    inline int GetWidth() { return width; }
    inline int GetHeight() { return height; }

    inline void SetWidth(unsigned int w) { width = w; }
    inline void SetHeight(unsigned int h) { height = h; }

    unsigned int GetTile(unsigned int layer, unsigned int x,unsigned int y);
    unsigned int GetLayerCount() ;

    inline std::vector<KeyValuePair> GetProperties() { return properties; };
    std::string GetPropertyValue(std::string name);
};

TmxMap* LoadTmxMap(std::string path);
std::string GetError();

}

#endif // __TMX_PARSER__
